Package: cmAnalysis
Title: Process and Visualise Concept Mapping Data
Version: 1.0.0
Authors@R: c(
    person(given = "Jos", family = "Hageman", email = "jos.hageman@wur.nl", role = c("aut", "cre")),
    person(given = "Jarl", family = "Kampen", email = "jarl.kampen@wur.nl", role = c("aut")))
Author: Jos Hageman [aut, cre],
  Jarl Kampen [aut]
Maintainer: Jos Hageman <jos.hageman@wur.nl>
Description: Processing and visualizing concept mapping data. Concept maps are versatile tools used across disciplines to enhance understanding, teaching, brainstorming, and information organization. The analysis of concept mapping data involves the sequential use of cluster analysis (for sorting participants and statements), multidimensional scaling (for positioning statements in a conceptual space), and visualization techniques, including point cluster maps and dendrograms.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: cluster, ggplot2, factoextra, clue, igraph, stringr, pheatmap
NeedsCompilation: no
Packaged: 2025-03-11 14:44:40 UTC; hagem011
Repository: CRAN
Date/Publication: 2025-03-13 12:50:05 UTC
Built: R 4.3.3; ; 2025-03-13 15:07:59 UTC; unix

Package: cmAnalysis
Title: Process and Visualise Concept Mapping Data
Version: 1.0.3
Authors@R: c(
    person(given = "Jos", family = "Hageman", email = "jos.hageman@wur.nl", role = c("aut", "cre")),
    person(given = "Jarl", family = "Kampen", email = "jarl.kampen@wur.nl", role = c("aut")))
Author: Jos Hageman [aut, cre],
  Jarl Kampen [aut]
Maintainer: Jos Hageman <jos.hageman@wur.nl>
Description: Concept maps are versatile tools used across disciplines to enhance understanding,
  teaching, brainstorming, and information organization. This package provides functions for
  processing and visualizing concept mapping data, involving the sequential use of cluster analysis 
  (for sorting participants and statements), multidimensional scaling (for positioning statements 
  in a conceptual space), and visualization techniques, including point cluster maps and 
  dendrograms. The methodology and its validity are discussed in Kampen, J.K., Hageman, J.A.,
  Breuer, M., & Tobi, H. (2025). "The validity of concept mapping: let's call a spade a spade." 
  Qual Quant. <doi:10.1007/s11135-025-02351-z>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: cluster, ggplot2, factoextra, clue, igraph, stringr, pheatmap
NeedsCompilation: no
Packaged: 2026-05-17 16:07:03 UTC; josha
Repository: CRAN
Date/Publication: 2026-05-18 20:40:08 UTC
Built: R 4.6.0; ; 2026-05-19 00:15:04 UTC; unix

Package: cmAnalysis
Title: Process and Visualise Concept Mapping Data
Version: 1.0.3
Authors@R: c(
    person(given = "Jos", family = "Hageman", email = "jos.hageman@wur.nl", role = c("aut", "cre")),
    person(given = "Jarl", family = "Kampen", email = "jarl.kampen@wur.nl", role = c("aut")))
Author: Jos Hageman [aut, cre],
  Jarl Kampen [aut]
Maintainer: Jos Hageman <jos.hageman@wur.nl>
Description: Concept maps are versatile tools used across disciplines to enhance understanding,
  teaching, brainstorming, and information organization. This package provides functions for
  processing and visualizing concept mapping data, involving the sequential use of cluster analysis 
  (for sorting participants and statements), multidimensional scaling (for positioning statements 
  in a conceptual space), and visualization techniques, including point cluster maps and 
  dendrograms. The methodology and its validity are discussed in Kampen, J.K., Hageman, J.A.,
  Breuer, M., & Tobi, H. (2025). "The validity of concept mapping: let's call a spade a spade." 
  Qual Quant. <doi:10.1007/s11135-025-02351-z>.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: cluster, ggplot2, factoextra, clue, igraph, stringr, pheatmap
NeedsCompilation: no
Packaged: 2026-05-17 16:07:03 UTC; josha
Repository: CRAN
Date/Publication: 2026-05-18 20:40:08 UTC
Built: R 4.5.2; ; 2026-05-18 22:28:12 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,conceptMap)
S3method(print,conceptMap)
S3method(summary,conceptMap)
export(abbreviateStatements)
export(checkConceptMapData)
export(conceptMapping)
export(createStatementOverview)
export(crossClusterMap)
export(numberOfSorters)
export(numberOfStatements)
export(printSorters)
export(selectStatements)
export(simulateCardData)
export(sorterMapping)
import(clue)
import(ggplot2)
importFrom(cluster,silhouette)
importFrom(factoextra,fviz_cluster)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(igraph,V)
importFrom(igraph,as_data_frame)
importFrom(igraph,cluster_leiden)
importFrom(igraph,gorder)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,layout_with_fr)
importFrom(igraph,strength)
importFrom(pheatmap,pheatmap)
importFrom(stats,chisq.test)
importFrom(stats,cmdscale)
importFrom(stats,dist)
importFrom(stringr,str_remove_all)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_squish)
importFrom(stringr,str_to_lower)

#' Select Significant Statements from Concept Map Data
#'
#' This function selects statements from a concept map dataset based on their significance in terms of co-occurrence.
#' It applies a chi-squared test on the co-occurrence matrix of the statements to identify those that are statistically significant
#' (i.e., those that co-occur more frequently than would be expected by chance).
#'
#' @param CMData A data frame containing concept map data. The data should have at least the following columns:
#'   - `statement`: The text of the statement.
#'   - `sorterID`: The identifier for the sorter.
#'   - `stackID`: The identifier for the stack.
#' @param significanceThreshold A numeric value representing the significance threshold for the chi-squared test.
#'   Statements with p-values less than this threshold are considered significant. Default is 0.05.
#' @param verbose Logical, if \code{TRUE}, additional information about the processing steps is printed to the console.
#' @param ... Additional arguments to be passed to the chi-squared test (optional).
#'
#' @return A data frame with the same structure as the input, but with non-significant statements removed (if any).
#'   If all statements are significant, the original data frame is returned unchanged.
#'
#' @seealso \code{\link{chisq.test}} for chi-squared test functionality.
#'
#' @importFrom stats chisq.test
#'
#' @examples
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.70, attributeWeights=c(1,1,1,1))
#'
#' # Subject the data to sorter cluster analysis
#' myCMDataBySorters <- sorterMapping(myCMData)
#'
#' # Select significant statements
#' mySelectedStatementsSorterCluster3 <- selectStatements(myCMDataBySorters[[1]])
#'
#' @export
selectStatements <- function(CMData, significanceThreshold = 0.05, verbose = TRUE, ...) {

  #check if the data frame is suitable for concept mapping
  if (!checkConceptMapData(CMData)) {
    stop("Object CMdata is not suitable for concept mapping.")
  }

  #add column StatementNumber for each unique statement
  CMData$StatementNumber <- as.numeric(factor(CMData$statement, levels=unique(CMData$statement)))

  #create overview of all statements
  allStatements <- createStatementOverview(CMData)

  #create co-occurence matrix for all statements
  coOccurrenceMatrix <- createCoOccurrenceMatrix(CMData)

  if (verbose) {
    cat(sprintf("Testing statements for uniformity.\n"))
  }

  pVals <- matrix(NA, nrow=nrow(allStatements), ncol=1)
  rownames(pVals) <- allStatements[,"Statement"]
  colnames(pVals) <- "p-value"

  for (i in 1:nrow(pVals)) {
    myTest <- chisq.test(coOccurrenceMatrix[i, -i])
    #myTest <- chisq.test(coOccurrenceMatrix[i, -i], ...)
    pVals[i] <- myTest$p.value
  }

  addSignificance <- function(p) {
    if (p < 0.001) {
      return("***")
    } else if (p < 0.01) {
      return("**")
    } else if (p < 0.05) {
      return("*")
    } else if (p < 0.1) {
      return(".")
    } else {
      return(" ")
    }
  }

  #output results of tests
  significanceInfo <- sapply(pVals, addSignificance)

  if (verbose) {
    cat(sprintf("Signif. codes:  0 '***' 0.001 '**' 0.01 '*' 0.05 '.' 0.1 ' ' 1\n"))
    cat(sprintf("                      ID  P-value\n"))

    for (i in 1:length(pVals)) {
      cat(sprintf("%2d. %20s  %.3e%s\n", allStatements$StatementNumber[i], allStatements$Statement[i], pVals[i], significanceInfo[i]))
    }
  }

  significant <- sum(pVals < significanceThreshold)
  nonSignificant <- sum(pVals >= significanceThreshold)

  # Display result
  if (verbose) {
    cat(sprintf("Significant statements in the sorter cluster: %d.\n", significant))
    cat(sprintf("Non-significant statements in the sorter cluster: %d.\n\n", nonSignificant))
  }

  if (nonSignificant > 0) {
    if (verbose) cat(sprintf("Overview of non-significant statements:\n"))
    idx <- which(pVals >= significanceThreshold)

    if (verbose) {
      for (jj in idx) {
        cat(sprintf("    %2d - %20s %.3e\n", jj, allStatements[jj, "Statement"], pVals[jj]))
      }
    }

    #remove the non-significant statements and return
    if (verbose) cat(sprintf("Removing %d non-significant statements from the data.\n", nonSignificant))
    idx <- which(CMData$statement %in% allStatements[idx, "Statement"])
    return(CMData[-idx,])
  } else {
    if (verbose) cat(sprintf("There are no non-significant statements to remove from the data.\n"))
  }

  return(CMData)
}

#' Cross-Cluster Mapping Between Concept Maps
#'
#' This function compares two concept maps by aligning their clustering results
#' and visualizing the correspondence between clusters. It identifies matches
#' between clusters from the two maps and highlights differences visually.
#'
#' @param conceptMap1 An object of class \code{"conceptMap"} representing the first concept map.
#' @param conceptMap2 An object of class \code{"conceptMap"} representing the second concept map.
#'
#' @details The function aligns clusters between two concept maps using an optimal matching
#' algorithm. It first creates a matching matrix based on the overlap between clusters
#' in the two maps. Then, it uses the Hungarian algorithm (via the \code{solve_LSAP} function
#' from the \code{clue} package) to find an optimal alignment of clusters.
#'
#' The output is a plot that shows the alignment of clusters from the two concept maps,
#' with connecting lines colored to indicate matches or mismatches. Statements not
#' clustered in both maps are highlighted in grey.
#'
#' @return The function does not return a value but generates a \code{ggplot2} visualization.
#'
#'
#' @seealso \code{\link[clue]{solve_LSAP}}, \code{\link[ggplot2]{ggplot}}
#'
#' @import ggplot2
#' @import clue
#'
#' @examples
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))
#'
#' # Subject the data to sorter cluster analysis
#' myCMDataBySorters <- sorterMapping(myCMData)
#'
#' # Subject sorter cluster 1 to concept mapping using default "network" method
#' myCMAnalysis1 <- conceptMapping(myCMDataBySorters[[1]])
#'
#' # Subject sorter cluster 3 to concept mapping using default "network" method
#' myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])
#'
#' # Visualise comparison of results of two sorter clusters
#' crossClusterMap(myCMAnalysis1, myCMAnalysis3)
#'
#' @export
crossClusterMap <- function(conceptMap1, conceptMap2) {

  if (!inherits(conceptMap1, "conceptMap")) {
    stop("Object conceptMap1 should be of class conceptMap")
  }

  if (!inherits(conceptMap2, "conceptMap")) {
    stop("Object conceptMap2 should be of class conceptMap")
  }

  #create object for common statements and how they are clustered in each concept map
  uniqueStatements <- union(conceptMap1$allStatements$Statement, conceptMap2$allStatements$Statement)
  clusteringResults <- matrix(NA, ncol=2, nrow=length(uniqueStatements))
  rownames(clusteringResults) <- uniqueStatements

  clusteringResults[names(conceptMap1$clusterResults), 1] <- conceptMap1$clusterResults
  clusteringResults[names(conceptMap2$clusterResults), 2] <- conceptMap2$clusterResults

  clusteringResult1 <- clusteringResults[,1]
  clusteringResult2 <- clusteringResults[,2]

  #Get the number of unique clusters in both results
  nClusters1 <- length(unique(clusteringResult1[!is.na(clusteringResult1)]))
  nClusters2 <- length(unique(clusteringResult2[!is.na(clusteringResult2)]))

  #Determine the size of the larger dimension to make the matrix square
  maxClusters <- max(nClusters1, nClusters2)

  #Create a square matrix with dimensions maxClusters x maxClusters
  matchingMatrix <- matrix(0, maxClusters, maxClusters)

  #Fill the matching matrix with actual matching counts from clustering results
  #Use the original dimensions (nClusters1 x nClusters2) for the filling process
  for (i in 1:nClusters1) {
    for (j in 1:nClusters2) {
      matchingMatrix[i, j] <- sum(clusteringResult1 == i & clusteringResult2 == j, na.rm = TRUE)
    }
  }

  #Now apply the solve_LSAP function on the square matching matrix
  optimalMatch <- clue::solve_LSAP(matchingMatrix, maximum=TRUE)

  clusteringResult2 <- optimalMatch[clusteringResult2]

  itemOrder1 <- order(clusteringResult1)
  itemOrder2 <- order(clusteringResult2)

  left2right <- sapply(1:length(itemOrder1), function(i) {
    which(itemOrder2 == i)
  })

  plotData <- data.frame( Item=1:length(clusteringResult1),
                          Cluster1=clusteringResult1,
                          Cluster2=clusteringResult2,
                          itemOrder1=itemOrder1,
                          itemOrder2=itemOrder2,
                          Left2right=left2right
  )

  myCols <- ifelse(plotData$Cluster1 == plotData$Cluster2, "forestgreen", "darkorange")

  mainTitle <- sprintf("Cross cluster map")

  #prepare plotting symbols
  myPCH1 <- as.character(plotData$Cluster1)
  myPCH1[is.na(myPCH1)] <- "X"
  myPCH2 <- as.character(plotData$Cluster2)
  myPCH2[is.na(myPCH2)] <- "X"

  #find if there are statements excluded from both clusterings
  idx <- which(is.na(plotData$Cluster1) & is.na(plotData$Cluster2))
  myCols[idx] <- "grey"

  p1 <- ggplot2::ggplot() +
    ggplot2::geom_text(data=plotData, ggplot2::aes(x=1 - 0.03, y=1:nrow(plotData),
                                                   label=rownames(plotData)[itemOrder1]), hjust=1, size=3) +
    ggplot2::geom_point(data=plotData[plotData$itemOrder1,], ggplot2::aes(x=1,
                                                                          y=1:nrow(plotData)), color=1, shape=myPCH1[plotData$itemOrder1], size=3)  +

    ggplot2::geom_text(data=plotData, ggplot2::aes(x=2 + 0.03, y=1:nrow(plotData),
                                                   label=rownames(plotData)[itemOrder2]), hjust=0, size=3 ) +
    ggplot2::geom_point(data=plotData[plotData$itemOrder2,], ggplot2::aes(x=2,
                                                                          y=1:nrow(plotData)), color=1, shape=myPCH2[plotData$itemOrder2], size=3) +

    ggplot2::geom_segment(data=plotData, ggplot2::aes(x=1, y=order(itemOrder1), xend=2, yend=.data$Left2right),
                          color=myCols, linewidth=0.7, alpha=0.7 ) +

    ggplot2::labs(x="Sorter clusters", y="Statement (arranged per cluster)", title=mainTitle,
                  shape="Cluster", color="Cluster") +
    ggplot2::theme_minimal() +
    ggplot2::scale_x_continuous(expand = ggplot2::expansion(mult = c(0.5, 0.5))) +
    ggplot2::theme(axis.text.x = ggplot2::element_blank(), axis.ticks.x = ggplot2::element_blank()) +
    ggplot2::theme(axis.text.y = ggplot2::element_blank(), axis.ticks.y = ggplot2::element_blank()) +
    ggplot2::theme(panel.grid.major = ggplot2::element_blank(),
                   panel.grid.minor = ggplot2::element_blank() )

  print(p1)
}

#' Generate Concept Mapping Network with Clustering
#'
#' This helper function constructs a network from a co-occurrence matrix, applies Leiden clustering,
#' and generates a plot of the network. It optimizes the resolution to achieve the specified number of clusters.
#'
#' @param coOccurrenceMatrix A numeric matrix representing the co-occurrence data.
#' @param numberOfClusters The number of clusters to form, or "auto" to automatically determine the optimal resolution.
#' @param main A string for the title of the plot.
#' @param verbose Logical, if \code{TRUE}, additional information about the processing steps is printed to the console.
#' @param backgroundColor The background color for the plot (default is "white").
#' @param resolution A resolution parameter for Leiden clustering (optional, defaults to `NULL`).
#'
#' @return A list with:
#'   - `networkPlot`: A ggplot object of the network plot with nodes and edges.
#'   - `networkNumberOfClusters`: The number of clusters formed in the network.
#'   - `networkClusterResults`: The cluster membership for each node.
#'
#' @importFrom igraph graph_from_adjacency_matrix cluster_leiden strength gorder as_data_frame layout_with_fr V
#' @importFrom ggplot2 ggplot geom_segment geom_point geom_text theme_void element_rect element_text labs
#' @noRd
conceptMappingNetwork <- function(coOccurrenceMatrix,
                                  numberOfClusters,
                                  main,
                                  verbose = TRUE,
                                  backgroundColor = "white",
                                  resolution = NULL) {

  graphObject <- igraph::graph_from_adjacency_matrix(coOccurrenceMatrix, mode = "undirected",
                                                     weighted = TRUE, diag = FALSE)

  if (numberOfClusters=="auto") {
    if (is.null(resolution)) {
      resolution <- stats::quantile(igraph::strength(graphObject))[2] / (igraph::gorder(graphObject) - 1)
    }
  } else {
    #get resolution value that matches with numberOfClusters
    if (verbose) {
      cat(sprintf("Opimizing resolution to get %d clusters in network...", numberOfClusters))
    }
    mySeq <- seq(from=0.1, to=50, by=0.01)
    numClusts <- numeric(length(mySeq))

    for (i in 1:length(mySeq)) {
      resolution <- mySeq[i]
      clusterResult <- igraph::cluster_leiden(graphObject, resolution=resolution, n_iterations = 10)
      numClusts[i] <- length(unique(clusterResult$membership))
      if (numClusts[i] >= numberOfClusters) {
        if (verbose) {
          cat(sprintf("done.\n"))
        }
        break
      }
    }
  }

  if (verbose) {
    cat(sprintf("Using Leiden algorithm with resolution: %.2f\n", resolution))
  }

  #get estimate for clusters
  clusterResult <- igraph::cluster_leiden(graphObject, resolution=resolution, n_iterations = 10)
  names(clusterResult$membership) <- clusterResult$names

  # Extract the edges and their weights for plotting
  edgeList <- igraph::as_data_frame(graphObject, what = "edges")

  # Create a layout for the network
  layoutDF <- as.data.frame(igraph::layout_with_fr(graphObject)) # Fruchterman-Reingold layout
  colnames(layoutDF) <- c("x", "y")
  # Check the node labels in layoutDF
  layoutDF$node <- igraph::V(graphObject)$name # Get node names from the graph object

  layoutDF$ID <- colnames(coOccurrenceMatrix)

  nodeNames <- layoutDF$node
  nodeCoords <- layoutDF[, c("x", "y")]

  # Initialize empty columns in the edgeList for from and to coordinates
  edgeList$from_x <- NA
  edgeList$from_y <- NA
  edgeList$to_x <- NA
  edgeList$to_y <- NA

  # Iteratively assign the 'from' and 'to' coordinates using a for loop and which function
  for (i in 1:nrow(edgeList)) {
    # Find the index of the 'from' node in the layoutDF
    fromIndex <- which(nodeNames == edgeList$from[i])
    toIndex <- which(nodeNames == edgeList$to[i])

    # Assign the 'from' coordinates
    edgeList$from_x[i] <- nodeCoords[fromIndex, "x"]
    edgeList$from_y[i] <- nodeCoords[fromIndex, "y"]

    # Assign the 'to' coordinates
    edgeList$to_x[i] <- nodeCoords[toIndex, "x"]
    edgeList$to_y[i] <- nodeCoords[toIndex, "y"]
  }

  #draw low weight lines first so they go to the back
  edgeList <- edgeList[order(edgeList$weight), ]

  if (backgroundColor=="black") {
    pointColor <- "blue"
    paperColor <- "black"
    textColor <- "white"
  } else {
    pointColor <- "blue"
    paperColor <- "white"
    textColor <- "black"
  }

  #Most different colors
  distinctiveColors <- c(
    "red",           #1
    "blue",          #2
    "green",         #3
    "yellow",        #4
    "purple",        #5
    "orange",        #6
    "cyan",          #7
    "pink",          #8
    "brown",         #9
    "gray",          #10
    "darkgreen",     #11
    "lightblue",     #12
    "gold",          #13
    "violet",        #14
    "darkgray"       #15
  )

  #the network plot
  theCMNetworkGraph <- ggplot2::ggplot() +
    ggplot2::geom_segment(data = edgeList, ggplot2::aes(x = .data$from_x, y = .data$from_y,
                                                        xend = .data$to_x, yend = .data$to_y,
                                                        linewidth = .data$weight, color=.data$weight)) +
    ggplot2::geom_point(data = layoutDF, ggplot2::aes(x = .data$x, y = .data$y), size = 5,
                        color = distinctiveColors[clusterResult$membership]) +
    ggplot2::geom_text(data = layoutDF, ggplot2::aes(x = .data$x, y = .data$y, label = .data$ID),
                       vjust = 1.5, size = 3, color=textColor) +
    ggplot2::theme_void() +
    ggplot2::theme(plot.background = ggplot2::element_rect(fill = paperColor, colour=paperColor),
                   panel.background = ggplot2::element_rect(fill = paperColor, colour=paperColor),
                   text = ggplot2::element_text(color = textColor),
                   legend.position = "none")

  theCMNetworkGraph <- theCMNetworkGraph + ggplot2::labs(title=paste("Network plot", main))


  return(list(networkPlot=theCMNetworkGraph,
              networkNumberOfClusters=length(unique(clusterResult$membership)),
              networkClusterResults=clusterResult$membership,
              resolution=resolution))
}

#' Simulate Card Sorting Data
#'
#' This function simulates card sorting data based on user-specified parameters,
#' such as the number of sorters, the probability of correct sorting, and the
#' weights for different card attributes.
#'
#' @param nSorters An integer specifying the number of sorters to simulate. Default is 40.
#' @param pCorrect A numeric value between 0 and 1 specifying the probability that
#'   a card is sorted correctly. Default is 0.95.
#' @param attributeWeights A numeric vector of length 4 specifying the weights
#'   for the card attributes (e.g., \code{color}, \code{suit}, \code{rank_picture},
#'   and \code{odd_even_picture}). Default is \code{c(1, 1, 1, 1)}.
#' @param verbose Logical, if \code{TRUE}, additional information about the processing
#'  steps is printed to the console.
#'
#' @details The function simulates a card sorting experiment where cards are
#'   sorted by multiple sorters based on one of four attributes. The probability
#'   of sorting a card correctly is determined by \code{pCorrect}, and errors are
#'   introduced randomly for each sorter. The attribute weights determine how many
#'   sorters focus on each attribute, and a warning is issued if the weights do
#'   not align with the total number of sorters.
#'
#'   The function returns a data frame containing simulated card sorting data
#'   for all sorters, including sorter IDs, card IDs, and assigned stacks.
#'
#' @return A data frame with columns:
#'   \describe{
#'     \item{\code{sorterID}}{A unique identifier for each sorter.}
#'     \item{\code{statement}}{The ID of the card being sorted.}
#'     \item{\code{stackID}}{The stack assigned to the card by the sorter.}
#'   }
#'
#' @examples
#' # Simulate data with default parameters:
#' set.seed(1)
#' myCMData <- simulateCardData()
#'
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))
#'
#' @seealso \code{\link[stats]{rbinom}}, \code{\link[base]{sample}}
#'
#' @export
simulateCardData <- function(nSorters = 40,
                             pCorrect = .95,
                             attributeWeights = c(1,1,1,1),
                             verbose = TRUE) {

  cardDeck <- structure(list(ID = c("Club2", "Club3", "Club4", "Club5", "Club6",
                                    "Club7", "Club8", "Club9", "Club10", "ClubJ", "ClubQ", "ClubK",
                                    "ClubA", "Spades2", "Spades3", "Spades4", "Spades5", "Spades6",
                                    "Spades7", "Spades8", "Spades9", "Spades10", "SpadesJ", "SpadesQ",
                                    "SpadesK", "SpadesA", "Diamonds2", "Diamonds3", "Diamonds4",
                                    "Diamonds5", "Diamonds6", "Diamonds7", "Diamonds8", "Diamonds9",
                                    "Diamonds10", "DiamondsJ", "DiamondsQ", "DiamondsK", "DiamondsA",
                                    "Hearts2", "Hearts3", "Hearts4", "Hearts5", "Hearts6", "Hearts7",
                                    "Hearts8", "Hearts9", "Hearts10", "HeartsJ", "HeartsQ", "HeartsK",
                                    "HeartsA", "Wildcard1", "Wildcard2"),
                             color = structure(c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L,
                                                 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 3L, 3L, 3L, 3L, 3L, 3L, 3L,
                                                 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L,
                                                 3L, 3L, 3L, 2L, 2L), levels = c("Black", "Wildcard", "Red"), class = "factor"),
                             suit = structure(c(1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L, 1L,
                                                1L, 1L, 1L, 5L, 5L, 5L, 5L, 5L, 5L, 5L, 5L, 5L, 5L, 5L, 5L,
                                                5L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 2L, 3L,
                                                3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 3L, 4L, 4L), levels = c("Club",
                                                                                                                    "Diamonds", "Hearts", "Wildcard", "Spades"), class = "factor"),
                             rank_picture = structure(c(2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L,
                                                        1L, 10L, 10L, 10L, 11L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, 1L,
                                                        10L, 10L, 10L, 11L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, 1L, 10L,
                                                        10L, 10L, 11L, 2L, 3L, 4L, 5L, 6L, 7L, 8L, 9L, 1L, 10L, 10L,
                                                        10L, 11L, 10L, 10L), levels = c("10", "2", "3", "4", "5",
                                                                                        "6", "7", "8", "9", "Picture", "Ace"), class = "factor"),
                             odd_even_picture = structure(c(1L, 2L, 1L, 2L, 1L, 2L, 1L,
                                                            2L, 1L, 3L, 3L, 3L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L,
                                                            3L, 3L, 3L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 3L, 3L,
                                                            3L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 2L, 1L, 3L, 3L, 3L, 2L,
                                                            3L, 3L), levels = c("Even", "Odd", "Picture"), class = "factor")), row.names = c(NA,
                                                                                                                                             -54L), class = "data.frame")

  if (verbose) {
    cat(sprintf("Simulating cardData.\n"))
    cat(sprintf("  Number of sorters: %d\n", nSorters))
    cat(sprintf("  Error probability: %f\n", pCorrect))
    cat(sprintf("  Attribute weights: %s\n", paste(attributeWeights, collapse = ", ")))
  }

  #get attribute labels from the cardDeck
  attributeLabels <- colnames(cardDeck)[-1]

  #determine number of sorters per attribute
  numbersOfSortersPerAttribute <- attributeWeights*(nSorters/sum(attributeWeights))

  #create sorterChoice vector
  sorterChoice <- unlist(as.vector(sapply(1:4, function(i) {
    rep(attributeLabels[i], numbersOfSortersPerAttribute[i])
  })))

  #report sorter's choices
  if (verbose) {
    cat(sprintf("  Number of sorters per sorting attribute:\n"))
    cat(table(sorterChoice))
    cat("\n")
  }

  #show warning in case number of sorters does not match with attribute weights
  if (sum(table(sorterChoice)) != nSorters) {
    warning(paste0("The number of requested sorters (", nSorters, ") is incompatible with the attribute weights.\nSetting number of sorters to ", sum(table(sorterChoice)), "."))
    nSorters <- sum(table(sorterChoice))
  }

  #run simulated data creation
  mySimulatedStack <- lapply(1:nSorters, function(jj) {
    #sort according to their choice
    myStackID <- cardDeck[, sorterChoice[jj]] #+1 is to point to correct column

    #we need to add some errors
    theSortErrors <- which(stats::rbinom(n=nrow(cardDeck), size=1, prob = (1-pCorrect))==1)

    #change some stackID's randomly (this means cards are misplaced)
    myStackID[theSortErrors] <- sample(levels(cardDeck[, sorterChoice[jj]]),
                                       size=length(theSortErrors), replace=TRUE)

    #store this sorter's work
    data.frame(sorterID=sprintf("Sorter%.2d", jj), statement=cardDeck[, 1], stackID=myStackID)
  })

  #unpack all the stack from all sorters and create one data set
  theSimulatedCardData <- do.call(rbind, mySimulatedStack)

  return(theSimulatedCardData)
}

#' Create Silhouette Plot for K-means Clustering
#'
#' This helper function generates a silhouette plot for K-means clustering
#' based on the provided co-occurrence matrix and range of cluster numbers.
#'
#' @param coOccurrenceMatrix A matrix of co-occurrence data to be clustered.
#' @param rangeNumberOfClusters A vector specifying the range of cluster numbers to evaluate (default is `2:15`).
#' @param numberOfKmeansRestarts Number of k-means restarts (default is 100).
#' @param main A string for the title of the silhouette plot.
#' @param forcedClusterNumber Optional fixed number of clusters to use (default is `NULL`).
#'
#' @return A list containing:
#'   - `silhouettePlot`: A ggplot object for the silhouette plot.
#'   - `silhouetteNumberOfClusters`: The optimal number of clusters determined by silhouette analysis.
#'   - `silhouetteClusterResults`: The cluster assignments for each data point.
#'
#' @importFrom cluster silhouette
#' @importFrom stats dist
#' @importFrom ggplot2 ggplot aes geom_line geom_point annotate labs theme_minimal
#' @noRd
createSilhouettePlot <- function(coOccurrenceMatrix,
                                 rangeNumberOfClusters = 2:15,
                                 numberOfKmeansRestarts = 100,
                                 main,
                                 forcedClusterNumber=NULL) {

  #Calculate the distance matrix for cluster k
  myDistance <- dist(coOccurrenceMatrix, method='euclidean')

  main <- paste("Kmeans", main)

  mySilhouetteResults <- doSilhouette(myDistance, rangeNumberOfClusters, numberOfKmeansRestarts, main, forcedClusterNumber)

  return(list(silhouettePlot=mySilhouetteResults$silhouettePlot,
              silhouetteNumberOfClusters=mySilhouetteResults$numberOfClusters,
              silhouetteClusterResults=mySilhouetteResults$clusterResults))
}

#' Create an Overview of Statements in Concept Map Data
#'
#' This function generates an overview of all unique statements in a given concept mapping dataset.
#' The function assigns a unique statement number to each distinct statement and returns a summary data frame.
#'
#' @param CMData A data frame containing concept map data. This must include a column named \code{"statement"}.
#'
#' @return A data frame with two columns: \code{StatementNumber} and \code{Statement}.
#'         Each row represents a unique statement with its corresponding number.
#'
#' @details
#' The function first checks if the provided dataset is suitable for concept mapping using the \cr
#' \code{checkConceptMapData} function. If the data is valid, it assigns a numeric statement number
#' to each distinct statement, ordered by their appearance in the dataset. The function then generates
#' an overview where each statement is paired with its corresponding statement number.
#'
#' @examples
#' # Example of valid data
#' CMData <- data.frame(
#'     sorterID = c("resp1", "resp1", "resp1", "resp2",
#'      "resp2", "resp2", "resp3", "resp3", "resp3"),
#'    statement = c("London", "Frankfurt", "Berlin", "London",
#'     "Frankfurt", "Berlin", "London", "Frankfurt", "Berlin"),
#'     stackID = c("capital city", "city", "capital city", 1, 2, 2, "A", "B", "A")
#' )
#'
#' # Create an overview of the statements
#' createStatementOverview(CMData)
#'
#' @export
createStatementOverview <- function(CMData) {

  #check if the data frame is suitable for concept mapping
  if (!checkConceptMapData(CMData)) {
    stop("Object CMdata is not suitable for concept mapping.")
  }

  CMData$StatementNumber <- as.numeric(factor(CMData$statement, levels=unique(CMData$statement)))

  #create overview of all statements
  statementOverview <- NULL
  for (i in 1:max(CMData$StatementNumber, na.rm=TRUE)) {
    idx <- which(CMData$StatementNumber == i)
    statementOverview <- rbind(statementOverview, data.frame(StatementNumber=i, Statement=CMData[idx, "statement"][1]))
  }

  return(statementOverview)
}

#' Plot a Concept Map
#'
#' This function generates visualizations for an object of class \code{conceptMap}.
#' Depending on the method used to create the concept map, different types of plots
#' (e.g., heatmap, silhouette, network, or CMDS plots) are available.
#'
#' @param x An object of class \code{conceptMap}. This object must contain
#'   plotting data and attributes specific to the method used for creating the concept map.
#' @param whichPlot A character string specifying which plot to display. Options depend on the
#'   method used to create the concept map:
#'   \describe{
#'     \item{\code{"all"}}{Displays all available plots for the given method.}
#'     \item{\code{"heatmap"}}{Displays the heatmap plot (if available).}
#'     \item{\code{"silhouette"}}{Displays the silhouette plot (if available).}
#'     \item{\code{"network"}}{Displays the network plot (if available).}
#'     \item{\code{"cmds"}}{Displays the CMDs plot (if available).}
#'   }
#'   The default is \code{"all"}.
#' @param ... arguments to be passed to methods
#'
#' @details The function behaves differently depending on the method used to create
#'   the concept map. The \code{conceptMap} object must include attributes such as
#'   \code{method} (e.g., "kmeans", "network", or "cmds") and the corresponding plot
#'   objects (e.g., \code{heatmapPlot}, \code{silhouettePlot}, etc.).
#'
#'   The following methods are supported:
#'   \itemize{
#'     \item \code{"kmeans"}: Supports \code{"heatmap"} and \code{"silhouette"} plots.
#'     \item \code{"network"}: Supports \code{"heatmap"} and \code{"network"} plots.
#'     \item \code{"cmds"}: Supports \code{"heatmap"}, \code{"silhouette"}, and \code{"cmds"} plots.
#'   }
#'
#' @return This function displays the specified plot(s) in the current graphical device.
#'
#' @examples
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))
#'
#' # Subject the data to sorter cluster analysis
#' myCMDataBySorters <- sorterMapping(myCMData)
#'
#' # Subject sorter cluster 3 to concept mapping using default "network" method
#' myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])
#'
#' # Visualise the concept map
#' plot(myCMAnalysis3)
#'
#' @export
plot.conceptMap <- function(x, whichPlot = "all", ...) {

  if (!inherits(x, "conceptMap")) {
    stop("Object x should be of class conceptMap")
  }

  if (x$method=="kmeans") {
    if (!whichPlot %in% c("all", "heatmap", "silhouette")) {
      stop(sprintf("Plot ID \"%s\" for method \"%s\" is not valid.\n", whichPlot, x$method))
    }
    if (whichPlot=="all") {
      plot(x$heatmapPlot$gtable)
      print(x$silhouettePlot)
    }
    if (whichPlot=="heatmap") {
      plot(x$heatmapPlot$gtable)
    }
    if (whichPlot=="silhouette") {
      print(x$silhouettePlot)
    }
  }

  if (x$method=="network") {
    if (!whichPlot %in% c("all", "heatmap", "network")) {
      stop(sprintf("Plot ID \"%s\" for method \"%s\" is not valid.\n", whichPlot, x$method))
    }
    if (whichPlot=="all") {
      plot(x$heatmapPlot$gtable)
      print(x$networkPlot)
    }
    if (whichPlot=="heatmap") {
      plot(x$heatmapPlot$gtable)
    }
    if (whichPlot=="network") {
      print(x$networkPlot)
    }
  }

  if (x$method=="cmds") {
    if (!whichPlot %in% c("all", "silhouette", "heatmap", "cmds")) {
      stop(sprintf("Plot ID \"%s\" for method \"%s\" is not valid.\n", whichPlot, x$method))
    }
    if (whichPlot=="all") {
      print(x$silhouettePlot)
      plot(x$heatmapPlot$gtable)
      print(x$cmdsPlot)
    }
    if (whichPlot=="silhouette") {
      print(x$silhouettePlot)
    }
    if (whichPlot=="heatmap") {
      plot(x$heatmapPlot$gtable)
    }
    if (whichPlot=="cmds") {
      print(x$cmdsPlot)
    }
  }
}

#' Create a Co-Occurrence Matrix from Concept Map Data
#'
#' This helper function generates a co-occurrence matrix, which represents how often each pair of statements
#' is grouped together across all sorters in the concept mapping dataset.
#'
#' @param CMData A data frame containing concept map data. The data must include the following columns:
#'        \code{sorterID}, \code{stackID}, and \code{StatementNumber}.
#'
#' @return A square matrix where rows and columns correspond to statements, and each element represents
#'         the number of times two statements were grouped together by sorters.
#'
#' @details
#' The resulting co-occurrence matrix is square and symmetric and includes row and column names derived from
#' the statements in the concept map data.
#'
#' @keywords internal
#' @noRd
createCoOccurrenceMatrix <- function(CMData) {

  #get the numbers on sorters and statements from the data
  allSorters <- unique(sort(CMData$sorterID))
  nSorters <- length(allSorters)
  nStatements <- length(unique(sort(CMData$statement)))

  rawData <- matrix(NA, nrow=nSorters, ncol=((nStatements - 1) * nStatements) / 2)

  #collect the data from each sorter
  for (i in 1:nSorters) {
    userSubset <- subset(CMData, CMData$sorterID == allSorters[i])

    #this matrix stores per sorter which statements go together
    #it is created for each sorter again
    tmp <- matrix(0, nrow=nStatements, ncol=nStatements)

    #now get each pile to see what is sorted together
    for (j in unique(userSubset$stackID)) {
      pileSubset <- subset(userSubset, userSubset$stackID == j)
      idx <- pileSubset$StatementNumber
      #setting it to 1 means they are grouped together
      tmp[idx, idx] <- 1
    }

    rawData[i,] <- tmp[lower.tri(tmp)]
  }

  #Calculate p
  p <- (1 + sqrt(1 + 8 * ncol(rawData))) / 2

  #Check if p is an integer, i.e., the number of columns should be a perfect square
  if (p != floor(p)) {
    stop(paste("The number of columns in is not a perfect square. Check your data."))
  }

  #Initialize the sum matrix with zeros
  coOccurrenceMatrix <- matrix(0, nrow=p, ncol=p)

  #Iterate over each row in the data
  for (h in 1:nrow(rawData)) {
    clusterMatrix <- matrix(0, nrow=p, ncol=p)

    #Fill the lower triangular part with the extracted vector
    clusterMatrix[lower.tri(clusterMatrix)] <- as.numeric(rawData[h, ])

    #Restore symmetry by adding the transpose
    clusterMatrix <- clusterMatrix + t(clusterMatrix)

    #Set the diagonal to 1s (since they were 1s in the original matrix)
    diag(clusterMatrix) <- 1

    #Add the row matrix to the sum matrix
    coOccurrenceMatrix <- coOccurrenceMatrix + clusterMatrix
  }

  overviewStatements <- createStatementOverview(CMData)

  colnames(coOccurrenceMatrix) <- overviewStatements$Statement
  rownames(coOccurrenceMatrix) <- overviewStatements$Statement

  return(coOccurrenceMatrix)
}

#' Cluster Sorters in Concept Mapping Data
#'
#' This function performs clustering of sorters in concept mapping data
#' based on their sorting behavior. It uses hierarchical clustering and allows
#' the automatic determination of the optimal number of clusters or a user-defined number.
#'
#' @param CMData A data frame containing concept mapping data. It must include the columns:
#'        \code{"sorterID"}, \code{"statement"}, and \code{"stackID"}.
#' @param numberOfSorterClusters Either a character string (\code{"auto"}) to automatically determine
#'        the optimal number of clusters or an integer specifying the desired number of clusters.
#' @param verbose Logical, if \code{TRUE}, additional information about the processing steps is printed to the console.
#' @param rangeNumberOfClusters A vector of integers specifying the range of clusters to evaluate
#'        when \cr \code{numberOfSorterClusters = "auto"}. Default is \code{2:15}.
#' @param graph Logical. If \code{TRUE}, plots the dendrogram and silhouette method results. Default is \code{TRUE}.
#'
#' @return A list of data frames, each representing the concept mapping data for a cluster of sorters.
#'         If only one cluster is found, the original \code{CMData} is returned.
#'
#' @details
#' This function clusters sorters based on their sorting behavior using hierarchical clustering
#' with Ward's method. If \code{numberOfSorterClusters = "auto"}, the silhouette method is used
#' to determine the optimal number of clusters within the range specified by \code{rangeNumberOfClusters}.
#'
#' Each cluster's data is validated for its suitability for concept mapping, and cluster-specific data
#' is returned as a list of data frames. Graphical output includes a dendrogram and silhouette plot
#' if \code{graph = TRUE}.
#'
#' @examples
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))
#'
#' # Subject the data to sorter cluster analysis
#' myCMDataBySorters <- sorterMapping(myCMData)
#'
#' # Subject the data to sorter cluster analysis with a predefined number of sorter clusters
#' myCMDataBySorters <- sorterMapping(myCMData, numberOfSorterClusters=2)
#'
#' @export
sorterMapping <- function(CMData,
                          numberOfSorterClusters = "auto",
                          verbose = TRUE,
                          rangeNumberOfClusters = 2:15,
                          graph = TRUE) {

  oldPar <- graphics::par(no.readonly = TRUE)
  on.exit(graphics::par(oldPar))

  #check if the data frame is suitable for concept mapping
  if (!checkConceptMapData(CMData)) {
    stop("Object CMdata is not suitable for concept mapping.")
  }

  #get the numbers on sorters and statements from the data
  allSorters <- unique(sort(CMData$sorterID))
  nSorters <- length(allSorters)
  nStatements <- length(unique(sort(CMData$statement)))

  if (is.character(numberOfSorterClusters)) {
    numberOfSorterClusters <- tolower(numberOfSorterClusters)
  }

  #make sure that the 1 cluster solution will not be used
  if (numberOfSorterClusters == "auto" & (1 %in% rangeNumberOfClusters)) {
    if (verbose) {
      cat(sprintf("Dropping cluster size of 1 in considered clusters."))
    }
    rangeNumberOfClusters <- setdiff(rangeNumberOfClusters, 1)
  }

  if (max(rangeNumberOfClusters) >= nSorters) {
    stop("Number of sorter clusters should be less than the number of sorters.")
  }

  #add column StatementNumber for each unique statement
  CMData$StatementNumber <- as.numeric(factor(CMData$statement, levels=unique(CMData$statement)))

  rawData <- matrix(NA, nrow=nSorters, ncol=((nStatements - 1) * nStatements) / 2)

  #collect the data from each individual user
  for (i in 1:nSorters) {
    userSubset <- subset(CMData, CMData$sorterID == allSorters[i])

    #this matrix stores per user which cards go together
    #create it again for each user
    tmp <- matrix(0, nrow=nStatements, ncol=nStatements)

    #now get each pile to see what is sorted together
    for (j in unique(userSubset$stackID)) {
      pileSubset <- subset(userSubset, userSubset$stackID == j)
      idx <- pileSubset$StatementNumber
      #setting it to 1 means they are grouped together
      tmp[idx, idx] <- 1
    }

    rawData[i,] <- tmp[lower.tri(tmp)]
  }

  #first we cluster the data, to see how many groups there are

  #calculate pairwise distances between rows
  theDistances <- stats::dist(rawData, method="euclidean")

  #Perform hierarchical clustering using Ward's method
  hc <- stats::hclust(theDistances, method="ward.D2")

  if (numberOfSorterClusters == "auto") {
    #Determine optimal number of clusters using Silhouette method from package cluster
    mySilhouetteWidth <- sapply(rangeNumberOfClusters, function(i) {
      sil <- cluster::silhouette(stats::cutree(hc, k=i), theDistances)
      return(mean(sil[, "sil_width"]))
    })

    optimalNumberofSorterClusters <- rangeNumberOfClusters[which.max(mySilhouetteWidth)]
    if (verbose) {
      cat(sprintf("Optimal number of sorter clusters based on Silhouette method: %d.\n",
                optimalNumberofSorterClusters))
    }

    if (graph) {
      #arrange two plots on one row
      graphics::par(mfrow=c(1, 2))
      plot(rangeNumberOfClusters, mySilhouetteWidth, type="b", pch=19,
           xlab="Number of clusters", ylab="Average silhouette width",
           main="Sorter Clusters by Silhouette Method")

      graphics::points(optimalNumberofSorterClusters, mySilhouetteWidth[which.max(mySilhouetteWidth)],
                       col="red", pch=15, cex=1.5)

      #Plot the dendrogram for total (mixed) data
      plot(hc,labels=FALSE, main="Dendrogram",
           xlab="Euclidean distance with Ward's Linkage", sub=NA)
      #visualize clusters
      stats::rect.hclust(hc, k=optimalNumberofSorterClusters, border="red")
      #return to one plot per figure
      graphics::par(mfrow=c(1, 1))
    }
  } else {
    optimalNumberofSorterClusters <- numberOfSorterClusters
    if (verbose) {
      cat(sprintf("Optimal number of sorter clusters (as set by user): %d.\n",
                optimalNumberofSorterClusters))
    }

    if (graph) {
      #Plot the dendrogram for total (mixed) data
      plot(hc,labels=FALSE, main="Dendrogram",
           xlab="Euclidean distance with Ward's Linkage", sub=NA)
      #visualize clusters
      stats::rect.hclust(hc, k=optimalNumberofSorterClusters, border="red")
    }
  }

  #now divide the raw data into sorter clusters and return a list of raw data frames
  clusteredSorters <- stats::cutree(hc, k=optimalNumberofSorterClusters)

  if (optimalNumberofSorterClusters > 1) {
    if (verbose) {
      cat(sprintf("Creating %d sorter clusters.\n", optimalNumberofSorterClusters))
    }
    sorterCMData <- list()
    for (i in 1:optimalNumberofSorterClusters) {
      if (verbose) {
        cat(sprintf("  Creating sorter cluster: %d\n", i))
      }
      idx <- CMData$sorterID %in% allSorters[clusteredSorters==i]
      sorterCMData[[i]] <- CMData[idx,]

      if (verbose) {
        cat(sprintf("    Number of sorters in this cluster: %d\n", numberOfSorters(sorterCMData[[i]])))
        cat(sprintf("    Data validity: "))

        #check resulting sorter data frame
        if (checkConceptMapData(sorterCMData[[i]])) {
          cat(sprintf("OK.\n"))
        } else {
          stop("Concept data not valid.")
        }
      }
    }
  } else {
    if (verbose) {
      cat(sprintf("Creating 1 sorter cluster.\n"))
    }
    sorterCMData <- CMData
  }

  return(sorterCMData)
}

#' Perform Concept Mapping Analysis
#'
#' This function conducts a concept mapping analysis on sorter data, producing a concept map
#' based on one of three clustering methods: k-means, network analysis, or classical multidimensional scaling (CMDS).
#'
#' @param CMData A data frame containing concept mapping data.
#'         It must include the columns:
#'        \code{"sorterID"}, \code{"statement"}, and \code{"stackID"}.
#' @param method A string specifying the clustering method to use. Options are:
#'        \code{"kmeans"}, \cr \code{"network"}, or \code{"cmds"}. Default is \code{"network"}.
#' @param numberOfClusters Either a character string (\code{"auto"}) to determine the optimal number of clusters
#'        or an integer specifying the desired number of clusters. Default is \code{"auto"}.
#' @param verbose Logical, if \code{TRUE}, additional information about the processing steps is printed to the console.
#' @param rangeNumberOfClusters A vector of integers specifying the range of clusters to evaluate
#'        when \cr \code{numberOfClusters = "auto"}. Default is \code{2:15}.
#' @param graph Logical. If \code{TRUE}, visualizes clustering results, including heatmaps and cluster-specific plots.
#'        Default is \code{FALSE}.
#' @param numberOfKmeansRestarts An integer specifying the number of restarts for k-means clustering.
#'        Only relevant if \code{method = "kmeans"}. Default is \code{100}.
#' @param backgroundColor A string specifying the background color of network plots. Default is \code{"black"}.
#' @param main A string specifying the title for plots. Default is \code{NULL}.
#' @param ... Additional arguments, such as \code{resolution}, which may be passed to specific clustering methods.
#'
#' @return An object of class \code{conceptMap}, containing:
#' \describe{
#'   \item{\code{allStatements}}{A data frame with statement numbers and text.}
#'   \item{\code{CMData}}{The original concept mapping data.}
#'   \item{\code{method}}{The clustering method used.}
#'   \item{\code{numberOfClusters}}{The number of clusters identified.}
#'   \item{\code{clusterResults}}{A vector indicating cluster assignments for each statement.}
#'   \item{\code{heatmapPlot}}{A heatmap visualizing co-occurrence patterns.}
#'   \item{\code{silhouettePlot}}{(If applicable) A silhouette plot for \code{"kmeans"} or \code{"cmds"}.}
#'   \item{\code{networkPlot}}{(If applicable) A network plot for \code{"network"}.}
#'   \item{\code{cmdsPlot}}{(If applicable) A CMDS plot for \code{"cmds"}.}
#' }
#'
#' @details
#' The function supports three methods for clustering:
#' \itemize{
#'   \item \code{"kmeans"}: Uses k-means clustering with an optional silhouette-based determination of cluster count.
#'   \item \code{"network"}: Generates a network plot using modularity-based clustering.
#'   \item \code{"cmds"}: Applies classical multidimensional scaling (CMDS) and clusters the results.
#' }
#'
#' Heatmaps are created for all methods, while additional visualizations depend on the chosen method and \code{graph} parameter.
#' 
#' @references
#' Kampen, J.K., Hageman, J.A., Breuer, M., & Tobi, H. (2025). The validity of concept mapping: let's call a spade a spade. Qual Quant. \doi{10.1007/s11135-025-02351-z}
#'
#' @examples
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))
#'
#' # Subject the data to sorter cluster analysis
#' myCMDataBySorters <- sorterMapping(myCMData)
#'
#' # Concept mapping on sorter cluster 3 using default "network" method
#' myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])
#'
#' # Concept mapping using default network method using 3 clusters
#' myCMAnalysis3b <- conceptMapping(myCMDataBySorters[[3]], numberOfCluster = 3)
#'
#' # Concept mapping using kmeans clustering and 3 clusters
#' myCMAnalysis3c <- conceptMapping(myCMDataBySorters[[3]], method = "kmeans",
#'   numberOfCluster = 3)
#'
#' @export
conceptMapping <- function(CMData,
                           method = "network",
                           numberOfClusters = "auto",
                           verbose = TRUE,
                           rangeNumberOfClusters = 2:15,
                           graph = FALSE,
                           numberOfKmeansRestarts = 100,
                           backgroundColor = "black",
                           main = NULL, ...) {

  #check if the data frame is suitable for concept mapping
  if (!checkConceptMapData(CMData)) {
    stop("Object CMdata is not suitable for concept mapping.")
  }

  if (!(method %in% c("kmeans", "network", "cmds"))) {
    stop(sprintf("Invalid method '%s' chosen.\n", method))
  }

  #extract elipsis
  args <- list(...)
  if ("resolution" %in% names(args)) {
    resolution <- args$resolution
  } else {
    resolution <- NULL
  }

  #add column StatementNumber for each unique statement
  CMData$StatementNumber <- as.numeric(factor(CMData$statement, levels=unique(CMData$statement)))

  #create overview of all statements
  allStatements <- createStatementOverview(CMData)

  #adjust cluster range to the maximum supported by available statements
  numberOfUniqueStatements <- length(unique(CMData$statement))
  maxNumberOfClusters <- numberOfUniqueStatements - 1
  adjustedRangeNumberOfClusters <- rangeNumberOfClusters[rangeNumberOfClusters <= maxNumberOfClusters]
  if (!identical(adjustedRangeNumberOfClusters, rangeNumberOfClusters)) {
    rangeNumberOfClusters <- adjustedRangeNumberOfClusters
    if (verbose) {
      cat(sprintf("Adjusted rangeNumberOfClusters to %s because there are %d unique statements (maximum clusters is %d).\n",
        paste(rangeNumberOfClusters, collapse = ", "), numberOfUniqueStatements, maxNumberOfClusters ))
    }
  }

  #create co-occurence matrix for all statements
  coOccurrenceMatrix <- createCoOccurrenceMatrix(CMData)

  conceptMap <- NULL
  #allocate memory for concept mapping results for each cluster k
  conceptMap$clusterResults <- rep(NA, length=nrow(allStatements))

  #Iterate over all clusters
  if (verbose) {
    cat(sprintf("Analyzing sorter cluster.\n"))
    cat(sprintf("Using method '%s' with number of clusters set to: '%s'.\n", method, as.character(numberOfClusters)))
  }

  #we always create a heatmap for visualisation purposes
  conceptMap$heatmapPlot <- createHeatmap(coOccurrenceMatrix, main)

  #1. method = kmeans
  if (method=="kmeans") {
    if (numberOfClusters=="auto") {
      silhouetteResults <- createSilhouettePlot(coOccurrenceMatrix,
                                                rangeNumberOfClusters,
                                                numberOfKmeansRestarts, main=main)
      conceptMap$silhouettePlot <- silhouetteResults$silhouettePlot
      conceptMap$numberOfClusters <- silhouetteResults$silhouetteNumberOfClusters
      conceptMap$clusterResults <- silhouetteResults$silhouetteClusterResults
    } else {
      silhouetteResults <- createSilhouettePlot(coOccurrenceMatrix,
                                                rangeNumberOfClusters,
                                                numberOfKmeansRestarts, main=main,
                                                forcedClusterNumber=numberOfClusters)
      conceptMap$silhouettePlot <- silhouetteResults$silhouettePlot
      conceptMap$numberOfClusters <- silhouetteResults$silhouetteNumberOfClusters
      conceptMap$clusterResults <- silhouetteResults$silhouetteClusterResults
    }

    if (graph) {
      plot(conceptMap$heatmapPlot$gtable)
      print(conceptMap$silhouettePlot)
    }
  }

  #2 method = network
  if (method=="network") {
    networkResults <- conceptMappingNetwork(coOccurrenceMatrix, numberOfClusters, main, verbose, backgroundColor, resolution)
    conceptMap$networkPlot <- networkResults$networkPlot
    conceptMap$numberOfClusters <- networkResults$networkNumberOfClusters
    conceptMap$clusterResults <- networkResults$networkClusterResults

    if (graph) {
      plot(conceptMap$heatmapPlot$gtable)
      print(conceptMap$networkPlot)
    }
  }

  #3 method = cmds
  if (method=="cmds") {
    cmdsResults <- createCMDSPlot(coOccurrenceMatrix, numberOfClusters, main=main,
                                  rangeNumberOfClusters, numberOfKmeansRestarts)
    conceptMap$cmdsPlot <- cmdsResults$cmdsPlot
    conceptMap$numberOfClusters <- cmdsResults$cmdsNumberOfClusters
    conceptMap$clusterResults <- cmdsResults$cmdsClusterResults
    conceptMap$silhouettePlot <- cmdsResults$cmdsSilhouettePlot

    if (graph) {
      print(conceptMap$silhouettePlot)
      plot(conceptMap$heatmapPlot$gtable)
      print(conceptMap$cmdsPlot)
    }
  }

  conceptMap$allStatements <- allStatements
  conceptMap$CMData <- CMData
  conceptMap$method <- method
  class(conceptMap) = "conceptMap"
  return(conceptMap)
}

#' Create Classical MDS Plot with Clustering
#'
#' This helper function performs Classical Multidimensional Scaling (CMDS) on a co-occurrence matrix
#' and generates a 2D plot with clusters. It also computes silhouette results based on the clustering.
#'
#' @param coOccurrenceMatrix A numeric matrix representing the co-occurrence data.
#' @param numberOfClusters The number of clusters to form, or "auto" to automatically determine the optimal number of clusters.
#' @param main A string for the title of the plot.
#' @param rangeNumberOfClusters A range of possible cluster numbers to evaluate for optimal clustering.
#' @param numberOfKmeansRestarts The number of restarts for k-means clustering.
#'
#' @return A list with:
#'   - `cmdsPlot`: A ggplot object of the CMDS plot with clusters.
#'   - `cmdsSilhouettePlot`: A ggplot object of the silhouette plot for clustering evaluation.
#'   - `cmdsNumberOfClusters`: The number of clusters identified.
#'   - `cmdsClusterResults`: The cluster assignment for each point.
#'
#' @importFrom stats dist cmdscale
#' @importFrom factoextra fviz_cluster
#' @importFrom ggplot2 theme_minimal ggtitle
#' @noRd
createCMDSPlot <- function(coOccurrenceMatrix,
                           numberOfClusters,
                           main,
                           rangeNumberOfClusters = 2:15,
                           numberOfKmeansRestarts = 100) {

  #Calculate the distance matrix for cluster k
  myDistance <- dist(coOccurrenceMatrix, method='euclidean')
  #Perform classical multidimensional scaling (MDS) in 2 dimensions
  CMcmds <- cmdscale(myDistance, k=2)

  #kmeans op CMcmds voor CMDSClusterNumber
  #warning("CMDS based clustering of statements is suboptimal method.")
  if (numberOfClusters=="auto") {
    numberOfClusters <- NULL
  }

  main <- paste("CMDS", main)

  mySilhouetteResults <- doSilhouette(CMcmds, rangeNumberOfClusters,
                                      numberOfKmeansRestarts, main, forcedClusterNumber=numberOfClusters )

  cmdsKmeans <- suppressWarnings(stats::kmeans(CMcmds, nstart=numberOfKmeansRestarts,
                                               iter.max=2000, centers=mySilhouetteResults$numberOfClusters))

  #Create a CMDS cluster plot for cluster k
  cmdsKMeansPlot <- factoextra::fviz_cluster(cmdsKmeans,
                                             data=as.data.frame(CMcmds),
                                             repel=TRUE,
                                             xlab="MDS Dimension 1",
                                             ylab="MDS Dimension 2") +
    ggplot2::theme_minimal() +
    ggplot2::ggtitle(paste("Point cluster map", main))

  return(list(cmdsPlot=cmdsKMeansPlot,
              cmdsSilhouettePlot=mySilhouetteResults$silhouettePlot,
              cmdsNumberOfClusters=mySilhouetteResults$numberOfClusters,
              cmdsClusterResults=mySilhouetteResults$clusterResults))
}

#' Create Co-occurrence Heatmap
#'
#' This helper function generates a heatmap for the provided co-occurrence matrix.
#' The heatmap is created using hierarchical clustering for both rows and columns.
#'
#' @param coOccurrenceMatrix A numeric matrix or data frame representing the co-occurrence data.
#' @param main A string for the title of the heatmap.
#'
#' @return A `pheatmap` object containing the heatmap plot.
#'
#' @importFrom pheatmap pheatmap
#' @importFrom stats dist
#' @noRd
createHeatmap <- function(coOccurrenceMatrix, main) {

  # Check if coOccurrenceMatrix is a data frame and convert to matrix if needed
  if (!is.matrix(coOccurrenceMatrix)) {
    if (is.data.frame(coOccurrenceMatrix)) {
      coOccurrenceMatrix <- as.matrix(coOccurrenceMatrix)
    } else {
      stop("Input must be a numeric matrix or data frame.")
    }
  }

  #create heatmap with row and column clustering
  pheatmapPlot <- pheatmap::pheatmap(
    coOccurrenceMatrix,
    color = grDevices::colorRampPalette(c("white", "red"))(50),
    clustering_distance_rows = "euclidean",
    clustering_distance_cols = "euclidean",
    clustering_method = "complete",
    show_rownames = TRUE,
    show_colnames = TRUE,
    fontsize_row = 8,
    fontsize_col = 8,
    legend = TRUE,
    silent = TRUE,
    main = paste("Co-occurrence Heatmap", main))

  return(pheatmapPlot)
}

#' Summary of a Concept Map Object
#'
#' This function provides a summary of a concept map object, including the number of statements,
#' sorters, clusters, and the distribution of statements across clusters.
#'
#' @param object An object of class \code{"conceptMap"} containing the results of concept mapping analysis
#' creating using the function \code{"conceptMapping"}.
#' @param ... arguments to be passed to methods
#'
#' @return This function does not return a value; it prints a summary of the concept map object to the console.
#'
#' @details
#' The function verifies that the input object is of class \code{"conceptMap"} and extracts key information from it.
#' It summarizes the number of statements, sorters, and clusters, and details the distribution of statements across
#' the identified clusters.
#'
#' @examples
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))
#'
#' # Subject the data to sorter cluster analysis
#' myCMDataBySorters <- sorterMapping(myCMData)
#'
#' # Subject sorter cluster 3 to concept mapping using default "network" method
#' myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])
#'
#' # Generate summary of concept map of sorter cluster 3
#' summary(myCMAnalysis3)
#'
#' @export
summary.conceptMap <- function(object, ...) {

  if (!inherits(object, "conceptMap")) {
    stop("'object' should be of class conceptMap")
  }

  cat(sprintf("Summary of the cluster:\n"))
  cat(sprintf("  Number of statements: %d\n", nrow(object$allStatements)))
  cat(sprintf("  Number of sorters: %d\n\n", length(unique(object$CMData$sorterID))))
  cat(sprintf("  Number of clusters in this sorter cluster: %d\n", object$numberOfClusters))

  for (k in sort(unique(object$clusterResults))) {
    idx <- which(object$clusterResults == k)
    cat(sprintf("  Cluster %d: %d statements\n", k, length(idx)))
  }
}

#' Perform Silhouette Analysis for Optimal Clustering
#'
#' This helper function computes the silhouette widths for a range of cluster numbers
#' and identifies the optimal number of clusters for clustering algorithms.
#'
#' @param myDf A numeric data frame or matrix for clustering.
#' @param rangeNumberOfClusters A vector specifying the number of clusters to evaluate (default is `2:15`).
#' @param numberOfKmeansRestarts Number of k-means restarts (default is 100).
#' @param main A string for the title of the silhouette plot.
#' @param forcedClusterNumber Optional fixed number of clusters to use (default is `NULL`).
#'
#' @return A list containing:
#'   - `silhouettePlot`: A ggplot object for the silhouette plot.
#'   - `numberOfClusters`: The optimal number of clusters.
#'   - `clusterResults`: The cluster assignments for each data point.
#'
#' @importFrom cluster silhouette
#' @importFrom ggplot2 ggplot aes geom_line geom_point annotate labs theme_minimal
#' @keywords internal
#' @noRd
doSilhouette <- function(myDf,
                         rangeNumberOfClusters = 2:15,
                         numberOfKmeansRestarts = 100,
                         main,
                         forcedClusterNumber=NULL) {

  mySilhouetteWidth <- NULL
  for (i in 1:length(rangeNumberOfClusters)) {
    myClus <- suppressWarnings(stats::kmeans(myDf, centers=rangeNumberOfClusters[i],
                                      iter.max=2000, nstart=numberOfKmeansRestarts))
    sil <- cluster::silhouette(myClus$cluster, dist(myDf))
    mySilhouetteWidth[i] <- mean(sil[, "sil_width"])
  }

  df <- data.frame("NumberOfClusters"=rangeNumberOfClusters,
                   "SilhouetteWidth"=mySilhouetteWidth)

  #Find the cluster number with the maximum silhouette width
  if (is.null(forcedClusterNumber)) {
    numberOfClusters <- rangeNumberOfClusters[which.max(mySilhouetteWidth)]
  } else {
    numberOfClusters <- forcedClusterNumber
  }

  silhouettePlot <-
    ggplot2::ggplot(df, ggplot2::aes(x=.data$NumberOfClusters, y=.data$SilhouetteWidth)) +
    ggplot2::geom_line() +
    ggplot2::geom_point(shape=19) +
    ggplot2::annotate("point", x = numberOfClusters, y = mySilhouetteWidth[which(rangeNumberOfClusters==numberOfClusters)],
                      color = "red", size = 3, shape=15) +
    ggplot2::labs(title=paste("Silhouette plot", main), x="Number of clusters", y="Average silhouette width") +
    ggplot2::theme_minimal()

  myKmeans <- suppressWarnings(stats::kmeans(myDf, nstart = numberOfKmeansRestarts,
                                             iter.max = 2000, centers = numberOfClusters))

  return(list(silhouettePlot = silhouettePlot,
              numberOfClusters = numberOfClusters,
              clusterResults = myKmeans$cluster))
}

#' Validate Concept Map Data
#'
#' Checks whether the provided data frame meets the requirements for concept map data.
#'
#' @param CMData A data frame containing concept map data. This data frame must include
#' the required columns: "sorterID", "statement", and "stackID".
#'
#' @return Returns \code{TRUE} if all checks pass. If any check fails, an error is raised
#' with a descriptive message.
#'
#' @details
#' This function performs the following checks on the input data:
#' \itemize{
#'   \item Verifies that \code{CMData} is a data frame.
#'   \item Ensures the presence of required columns: \code{"sorterID"}, \code{"statement"}, and \code{"stackID"}.
#'   \item Confirms that there are at least 2 unique values in the \code{"stackID"} column.
#'   \item Confirms that there are at least 2 unique values in the \code{"sorterID"} column.
#'   \item Confirms that there are at least 2 unique values in the \code{"statement"} column.
#' }
#'
#' @examples
#' # Example of valid data
#' validData <- data.frame(
#'   sorterID = c("resp1", "resp1", "resp1", "resp2",
#'    "resp2", "resp2", "resp3", "resp3", "resp3"),
#'   statement = c("London", "Frankfurt", "Berlin", "London",
#'   "Frankfurt", "Berlin", "London", "Frankfurt", "Berlin"),
#'   stackID = c("capital city", "city", "capital city", 1, 2, 2, "A", "B", "A")
#' )
#' checkConceptMapData(validData) # Should return TRUE
#'
#' # Example of invalid data (missing columns)
#' invalidData <- data.frame(
#'   sorterID = c(1, 2),
#'   stackID = c(1, 2)
#' )
#' # checkConceptMapData(invalidData) # Will return False as an error
#'
#' @export
checkConceptMapData <- function(CMData) {
  #Check if CMData is a data frame
  if (!is.data.frame(CMData)) {
    stop("Object CMData must be a data frame.")
  }

  #Check if the required columns are present
  requiredColumns <- c("sorterID", "statement", "stackID")
  missingColumns <- setdiff(requiredColumns, colnames(CMData))

  if (length(missingColumns) > 0) {
    stop(paste(
      "The following required columns are missing from Object CMData:",
      paste(missingColumns, collapse=", ")
    ))
  }

  #Check if there are at least 2 unique stacks (sorterID, stackID combinations)
  if (nrow(unique(CMData[, c("sorterID", "stackID")])) < 2) {
    stop("Object CMData must contain at least 2 unique stacks.")
  }

  #Check if there are at least 2 unique sorterID's
  if (length(unique(CMData$sorterID)) < 2) {
    stop("Data must contain at least 2 unique users.")
  }

  #Check if there are at least 2 unique statements
  if (length(unique(sort(CMData$statement))) < 2) {
    stop("Data must contain at least 2 unique statements.")
  }

  return(TRUE)
}

#' Print a Concept Map Object
#'
#' This function prints a detailed summary of a concept map object, including information about
#' statements, sorters, clusters, and the specific statements within selected clusters.
#'
#' @param x An object of class \code{"conceptMap"} containing the results of concept mapping analysis
#' creating using the function \code{\link[cmAnalysis]{conceptMapping}}.
#'
#' @param whichCluster A vector of cluster numbers to display. If \code{NULL} (default), all clusters are displayed.
#'
#' @param ... arguments to be passed to methods
#'
#' @return This function does not return a value; it prints the details of the concept map to the console.
#'
#' @details
#' The function first checks if the input object is of class \code{"conceptMap"} and validates the
#' requested clusters (if specified). It provides an overview of the number of statements, sorters,
#' and clusters. For each requested cluster, the function lists the statements included in that cluster.
#'
#' @examples
#' # Simulate data with custom parameters:
#' set.seed(1)
#' myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))
#'
#' # Subject the data to sorter cluster analysis
#' myCMDataBySorters <- sorterMapping(myCMData)
#'
#' # Subject sorter cluster 3 to concept mapping using default "network" method
#' myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])
#'
#' # Print content of concept map of sorter cluster 3
#' print(myCMAnalysis3)
#'
#' @export
print.conceptMap <- function(x, whichCluster = NULL, ...) {

  if (!inherits(x, "conceptMap")) {
    stop("Object x should be of class conceptMap")
  }

  cat(sprintf("Summary of the cluster:\n"))
  cat(sprintf("Number of statements: %d\n", nrow(x$allStatements)))
  cat(sprintf("Number of sorters: %d\n", length(unique(x$CMData$sorterID))))
  for (jj in 1:length(unique(x$CMData$sorterID))) {
    cat(sprintf("  %2d - %s\n", jj, unique(x$CMData$sorterID)[jj] ))
  }
  cat(sprintf("Number of clusters in this sorter cluster: %d\n", x$numberOfClusters))

  #check which cluster we want
  if (is.null(whichCluster)) {
    whichCluster <- 1:x$numberOfClusters
  } else {
    if (max(whichCluster) > x$numberOfClusters) {
      stop(sprintf("Requested cluster %d does not exist in the sorter cluster.", max(whichCluster)))
    }
  }

  for (k in whichCluster) {
    idx <- which(x$clusterResults == k)
    cat(sprintf("  Cluster %d: %d statements\n", k, length(idx)))
    for (jj in idx) {
      cat(sprintf("    %2d - %s\n", jj, x$allStatements[jj, "Statement"]))
    }
  }
}

#' Abbreviate Statements in Concept Map Data
#'
#' This function processes the "statement" column of a data frame containing concept map data by converting each
#' statement to lowercase, removing stopwords, and truncating the statement to a specified maximum length.
#' It allows for the abbreviation of long statements while maintaining their core meaning by removing unnecessary words.
#'
#' @param CMData A data frame containing concept map data. The data should have at least the following column:
#'   - `statement`: The text of the statement to be abbreviated.
#'
#' @param max_length An integer specifying the maximum number of characters for the abbreviated statement. Default is 30.
#'
#' @details This function performs several preprocessing steps on the "statement" column:
#'   - Converts statements to lowercase.
#'   - Removes punctuation and stopwords from the statements.
#'   - Truncates statements to a specified maximum length.
#'   - Removes any rows with empty statements after processing.
#'
#' Stopwords are predefined and include common English words (e.g., "the", "and", "is", "a", etc.) that do not contribute
#' much meaning to the core idea of the statement.
#'
#' @return A data frame with the same structure as the input, but with an updated "statement" column containing the
#' abbreviated statements.
#'
#' @importFrom stringr str_to_lower str_replace_all str_squish str_remove_all
#'
#' @examples
#' # Create a sample data frame with concept map data
#' conceptMapData <- data.frame(
#'    id = c(1, 2, 3),
#'     statement = c(
#'        "The quick brown fox jumps over the lazy dog",
#'        "This is a simple concept map example",
#'        "Data science involves analyzing datasets"
#'     )
#' )
#'
#' # Apply the abbreviateStatements function with a maximum length of 20
#' result <- abbreviateStatements(conceptMapData, max_length = 20)
#'
# Display the resulting data frame
#' print(result)
#'
#' @export
abbreviateStatements <- function(CMData, max_length = 30) {

  # Step 1: Define the list of stopwords directly in the script
  stopwords <- structure(list(
    "la", "ns", "ef", "al", "lol", "shit", "fucking", "fuck", "c", "im",
    "gt", "s", "t", "u", "k", "us", "d", "m", "n", "j", "h", "e", "re",
    "may", "ive", "r", "ese", "aj", "y", "hes", "shes", "doesnt", "didnt",
    "dont", "isnt", "arent", "wasnt", "werent", "hasnt", "havent",
    "hadnt", "wont", "wouldnt", "cant", "couldnt", "shouldnt", "might",
    "mightnt", "mustnt", "whats", "whos", "wheres", "whens", "whys",
    "hows", "youre", "youll", "rulescarefully", "please", "read", "join",
    "can", "will", "ve", "just", "like", "make", "think", "know", "get",
    "got", "one", "time", "going", "see", "say", "said", "want", "also",
    "way", "even", "take", "saying", "still", "really", "much", "many",
    "back", "go", "well", "look", "good", "need", "first", "two", "say",
    "said", "let", "right", "gonna", "something", "thing", "things", "lot",
    "sure", "maybe", "yes", "no", "okay", "hello", "hi", "hey", "thanks",
    "thank", "sorry", "ok", "yeah", "yes", "nope", "new", "old", "great",
    "better", "best", "awesome", "amazing", "cool", "ll", "reddit", "op",
    "sub", "ing", "etc", "don", "thats", "wasn", "saw", "went", "didn",
    "bring", "nk", "f", "theyre", "ese", "i", "me", "my", "myself", "we",
    "our", "ours", "ourselves", "you", "your", "yours", "yourself",
    "yourselves", "he", "him", "his", "himself", "she", "her", "hers",
    "herself", "it", "its", "itself", "they", "them", "their", "theirs",
    "themselves", "what", "which", "who", "whom", "this", "that", "these",
    "those", "am", "is", "are", "was", "were", "be", "been", "being",
    "have", "has", "had", "having", "do", "does", "did", "doing", "a",
    "an", "the", "and", "but", "if", "or", "because", "as", "until",
    "while", "of", "at", "by", "for", "with", "about", "against",
    "between", "into", "through", "during", "before", "after", "above",
    "below", "to", "from", "up", "down", "in", "out", "on", "off",
    "over", "under", "again", "further", "then", "once", "here",
    "there", "when", "where", "why", "how", "all", "any", "both",
    "each", "few", "more", "most", "other", "some", "such", "no",
    "nor", "not", "only", "own", "same", "so", "than", "too", "very",
    "s", "t", "can", "will", "just", "don", "should", "now", "ever",
    "used", "never", "told", "st", "nd", "rd", "th"), .Names = NULL)

  # Step 2: Convert the stopwords into a regular expression pattern
  stopwords_pattern <- paste0("\\b(", paste(stopwords, collapse = "|"), ")\\b")

  # Step 3: Process the "statement" column of CMData
  # Step-by-step processing without pipes
  CMData$oriStatement <- CMData$statement  # Keep the original statement as "oriStatement"

  # Convert the statement to lowercase
  CMData$statement <- stringr::str_to_lower(CMData$statement)

  # Remove punctuation from the statement
  CMData$statement <- stringr::str_replace_all(CMData$statement, "[[:punct:]]", "")

  # Remove stopwords and extra spaces
  CMData$statement <- stringr::str_squish(stringr::str_remove_all(CMData$statement, stopwords_pattern))

  # Restrict the length to max_length characters
  CMData$statement <- substr(CMData$statement, 1, max_length)

  # Remove rows with empty statements
  CMData <- CMData[CMData$statement != "", ]
  # Step 4: Return the modified dataframe
  return(CMData)
}

#' Print and Return Unique Sorters in Concept Map Data
#'
#' This function retrieves and optionally prints the unique sorters (users) in a given concept mapping dataset.
#'
#' @param CMData A data frame containing concept map data. This must include a column named \code{"sorterID"}.
#' @param verbose A logical, if \code{TRUE}, the function will print the list of unique sorters to the console.
#'
#' @return A vector of unique sorter IDs.
#'
#' @details
#' The function first checks if the provided dataset is suitable for concept mapping using the \cr
#' \code{checkConceptMapData} function. If the data is valid, it retrieves the unique sorter IDs
#' from the \code{sorterID} column. If \code{verbose = TRUE}, the function prints the sorter IDs.
#'
#' @examples
#' # Example of valid data
#' CMData <- data.frame(
#'     sorterID = c("resp1", "resp1", "resp1", "resp2",
#'      "resp2", "resp2", "resp3", "resp3", "resp3"),
#'    statement = c("London", "Frankfurt", "Berlin", "London",
#'     "Frankfurt", "Berlin", "London", "Frankfurt", "Berlin"),
#'     stackID = c("capital city", "city", "capital city", 1, 2, 2, "A", "B", "A")
#' )
#'
#' # Retrieve unique sorters without printing
#' printSorters(CMData, verbose = FALSE)
#'
#' # Retrieve and print unique sorters to console
#' printSorters(CMData)
#'
#' @export
printSorters <- function(CMData, verbose = TRUE) {

  #check if the data frame is suitable for concept mapping
  if (!checkConceptMapData(CMData)) {
    stop("Object CMdata is not suitable for concept mapping.")
  }

  allSorters <- unique(sort(CMData$sorterID))

  if (verbose) {
    cat(sprintf("Sorters in this cluster:\n"))
    for (i in allSorters) {
      cat(sprintf("  %s\n", i))
    }
  }

  return(allSorters)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/selectStatements.R
\name{selectStatements}
\alias{selectStatements}
\title{Select Significant Statements from Concept Map Data}
\usage{
selectStatements(CMData, significanceThreshold = 0.05, verbose = TRUE, ...)
}
\arguments{
\item{CMData}{A data frame containing concept map data. The data should have at least the following columns:
\itemize{
\item \code{statement}: The text of the statement.
\item \code{sorterID}: The identifier for the sorter.
\item \code{stackID}: The identifier for the stack.
}}

\item{significanceThreshold}{A numeric value representing the significance threshold for the chi-squared test.
Statements with p-values less than this threshold are considered significant. Default is 0.05.}

\item{verbose}{Logical, if \code{TRUE}, additional information about the processing steps is printed to the console.}

\item{...}{Additional arguments to be passed to the chi-squared test (optional).}
}
\value{
A data frame with the same structure as the input, but with non-significant statements removed (if any).
If all statements are significant, the original data frame is returned unchanged.
}
\description{
This function selects statements from a concept map dataset based on their significance in terms of co-occurrence.
It applies a chi-squared test on the co-occurrence matrix of the statements to identify those that are statistically significant
(i.e., those that co-occur more frequently than would be expected by chance).
}
\examples{
# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.70, attributeWeights=c(1,1,1,1))

# Subject the data to sorter cluster analysis
myCMDataBySorters <- sorterMapping(myCMData)

# Select significant statements
mySelectedStatementsSorterCluster3 <- selectStatements(myCMDataBySorters[[1]])

}
\seealso{
\code{\link{chisq.test}} for chi-squared test functionality.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulateCardData.R
\name{simulateCardData}
\alias{simulateCardData}
\title{Simulate Card Sorting Data}
\usage{
simulateCardData(
  nSorters = 40,
  pCorrect = 0.95,
  attributeWeights = c(1, 1, 1, 1),
  verbose = TRUE
)
}
\arguments{
\item{nSorters}{An integer specifying the number of sorters to simulate. Default is 40.}

\item{pCorrect}{A numeric value between 0 and 1 specifying the probability that
a card is sorted correctly. Default is 0.95.}

\item{attributeWeights}{A numeric vector of length 4 specifying the weights
for the card attributes (e.g., \code{color}, \code{suit}, \code{rank_picture},
and \code{odd_even_picture}). Default is \code{c(1, 1, 1, 1)}.}

\item{verbose}{Logical, if \code{TRUE}, additional information about the processing
steps is printed to the console.}
}
\value{
A data frame with columns:
\describe{
\item{\code{sorterID}}{A unique identifier for each sorter.}
\item{\code{statement}}{The ID of the card being sorted.}
\item{\code{stackID}}{The stack assigned to the card by the sorter.}
}
}
\description{
This function simulates card sorting data based on user-specified parameters,
such as the number of sorters, the probability of correct sorting, and the
weights for different card attributes.
}
\details{
The function simulates a card sorting experiment where cards are
sorted by multiple sorters based on one of four attributes. The probability
of sorting a card correctly is determined by \code{pCorrect}, and errors are
introduced randomly for each sorter. The attribute weights determine how many
sorters focus on each attribute, and a warning is issued if the weights do
not align with the total number of sorters.

The function returns a data frame containing simulated card sorting data
for all sorters, including sorter IDs, card IDs, and assigned stacks.
}
\examples{
# Simulate data with default parameters:
set.seed(1)
myCMData <- simulateCardData()

# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))

}
\seealso{
\code{\link[stats]{rbinom}}, \code{\link[base]{sample}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary.conceptMap.R
\name{summary.conceptMap}
\alias{summary.conceptMap}
\title{Summary of a Concept Map Object}
\usage{
\method{summary}{conceptMap}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"conceptMap"} containing the results of concept mapping analysis
creating using the function \code{"conceptMapping"}.}

\item{...}{arguments to be passed to methods}
}
\value{
This function does not return a value; it prints a summary of the concept map object to the console.
}
\description{
This function provides a summary of a concept map object, including the number of statements,
sorters, clusters, and the distribution of statements across clusters.
}
\details{
The function verifies that the input object is of class \code{"conceptMap"} and extracts key information from it.
It summarizes the number of statements, sorters, and clusters, and details the distribution of statements across
the identified clusters.
}
\examples{
# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))

# Subject the data to sorter cluster analysis
myCMDataBySorters <- sorterMapping(myCMData)

# Subject sorter cluster 3 to concept mapping using default "network" method
myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])

# Generate summary of concept map of sorter cluster 3
summary(myCMAnalysis3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abbreviateStatements.R
\name{abbreviateStatements}
\alias{abbreviateStatements}
\title{Abbreviate Statements in Concept Map Data}
\usage{
abbreviateStatements(CMData, max_length = 30)
}
\arguments{
\item{CMData}{A data frame containing concept map data. The data should have at least the following column:
\itemize{
\item \code{statement}: The text of the statement to be abbreviated.
}}

\item{max_length}{An integer specifying the maximum number of characters for the abbreviated statement. Default is 30.}
}
\value{
A data frame with the same structure as the input, but with an updated "statement" column containing the
abbreviated statements.
}
\description{
This function processes the "statement" column of a data frame containing concept map data by converting each
statement to lowercase, removing stopwords, and truncating the statement to a specified maximum length.
It allows for the abbreviation of long statements while maintaining their core meaning by removing unnecessary words.
}
\details{
This function performs several preprocessing steps on the "statement" column:
\itemize{
\item Converts statements to lowercase.
\item Removes punctuation and stopwords from the statements.
\item Truncates statements to a specified maximum length.
\item Removes any rows with empty statements after processing.
}

Stopwords are predefined and include common English words (e.g., "the", "and", "is", "a", etc.) that do not contribute
much meaning to the core idea of the statement.
}
\examples{
# Create a sample data frame with concept map data
conceptMapData <- data.frame(
   id = c(1, 2, 3),
    statement = c(
       "The quick brown fox jumps over the lazy dog",
       "This is a simple concept map example",
       "Data science involves analyzing datasets"
    )
)

# Apply the abbreviateStatements function with a maximum length of 20
result <- abbreviateStatements(conceptMapData, max_length = 20)

print(result)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numberOfSorters.R
\name{numberOfSorters}
\alias{numberOfSorters}
\title{Count the Number of Sorters in Concept Map Data}
\usage{
numberOfSorters(CMData, verbose = TRUE)
}
\arguments{
\item{CMData}{A data frame containing concept map data. This must include a column named \code{"sorterID"}.}

\item{verbose}{A logical, if \code{TRUE}, the function will print the number of sorters.}
}
\value{
An integer representing the number of unique sorters in the dataset.
}
\description{
This function calculates the number of unique sorters (users) in a given concept mapping dataset.
}
\details{
The function first checks if the provided dataset is suitable for concept mapping using the \cr
\code{checkConceptMapData} function. If the data is valid, it calculates and returns the number
of unique \code{sorterID}s.
}
\examples{
# Example of valid data
CMData <- data.frame(
    sorterID = c("resp1", "resp1", "resp1", "resp2",
     "resp2", "resp2", "resp3", "resp3", "resp3"),
   statement = c("London", "Frankfurt", "Berlin", "London",
    "Frankfurt", "Berlin", "London", "Frankfurt", "Berlin"),
    stackID = c("capital city", "city", "capital city", 1, 2, 2, "A", "B", "A")
)

# Count the number of sorters silently
numberOfSorters(CMData, verbose = FALSE)

# Count the number of sorters with message
numberOfSorters(CMData)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/createStatementOverview.R
\name{createStatementOverview}
\alias{createStatementOverview}
\title{Create an Overview of Statements in Concept Map Data}
\usage{
createStatementOverview(CMData)
}
\arguments{
\item{CMData}{A data frame containing concept map data. This must include a column named \code{"statement"}.}
}
\value{
A data frame with two columns: \code{StatementNumber} and \code{Statement}.
Each row represents a unique statement with its corresponding number.
}
\description{
This function generates an overview of all unique statements in a given concept mapping dataset.
The function assigns a unique statement number to each distinct statement and returns a summary data frame.
}
\details{
The function first checks if the provided dataset is suitable for concept mapping using the \cr
\code{checkConceptMapData} function. If the data is valid, it assigns a numeric statement number
to each distinct statement, ordered by their appearance in the dataset. The function then generates
an overview where each statement is paired with its corresponding statement number.
}
\examples{
# Example of valid data
CMData <- data.frame(
    sorterID = c("resp1", "resp1", "resp1", "resp2",
     "resp2", "resp2", "resp3", "resp3", "resp3"),
   statement = c("London", "Frankfurt", "Berlin", "London",
    "Frankfurt", "Berlin", "London", "Frankfurt", "Berlin"),
    stackID = c("capital city", "city", "capital city", 1, 2, 2, "A", "B", "A")
)

# Create an overview of the statements
createStatementOverview(CMData)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sorterMapping.R
\name{sorterMapping}
\alias{sorterMapping}
\title{Cluster Sorters in Concept Mapping Data}
\usage{
sorterMapping(
  CMData,
  numberOfSorterClusters = "auto",
  verbose = TRUE,
  rangeNumberOfClusters = 2:15,
  graph = TRUE
)
}
\arguments{
\item{CMData}{A data frame containing concept mapping data. It must include the columns:
\code{"sorterID"}, \code{"statement"}, and \code{"stackID"}.}

\item{numberOfSorterClusters}{Either a character string (\code{"auto"}) to automatically determine
the optimal number of clusters or an integer specifying the desired number of clusters.}

\item{verbose}{Logical, if \code{TRUE}, additional information about the processing steps is printed to the console.}

\item{rangeNumberOfClusters}{A vector of integers specifying the range of clusters to evaluate
when \cr \code{numberOfSorterClusters = "auto"}. Default is \code{2:15}.}

\item{graph}{Logical. If \code{TRUE}, plots the dendrogram and silhouette method results. Default is \code{TRUE}.}
}
\value{
A list of data frames, each representing the concept mapping data for a cluster of sorters.
If only one cluster is found, the original \code{CMData} is returned.
}
\description{
This function performs clustering of sorters in concept mapping data
based on their sorting behavior. It uses hierarchical clustering and allows
the automatic determination of the optimal number of clusters or a user-defined number.
}
\details{
This function clusters sorters based on their sorting behavior using hierarchical clustering
with Ward's method. If \code{numberOfSorterClusters = "auto"}, the silhouette method is used
to determine the optimal number of clusters within the range specified by \code{rangeNumberOfClusters}.

Each cluster's data is validated for its suitability for concept mapping, and cluster-specific data
is returned as a list of data frames. Graphical output includes a dendrogram and silhouette plot
if \code{graph = TRUE}.
}
\examples{
# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))

# Subject the data to sorter cluster analysis
myCMDataBySorters <- sorterMapping(myCMData)

# Subject the data to sorter cluster analysis with a predefined number of sorter clusters
myCMDataBySorters <- sorterMapping(myCMData, numberOfSorterClusters=2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.conceptMap.R
\name{plot.conceptMap}
\alias{plot.conceptMap}
\title{Plot a Concept Map}
\usage{
\method{plot}{conceptMap}(x, whichPlot = "all", ...)
}
\arguments{
\item{x}{An object of class \code{conceptMap}. This object must contain
plotting data and attributes specific to the method used for creating the concept map.}

\item{whichPlot}{A character string specifying which plot to display. Options depend on the
method used to create the concept map:
\describe{
\item{\code{"all"}}{Displays all available plots for the given method.}
\item{\code{"heatmap"}}{Displays the heatmap plot (if available).}
\item{\code{"silhouette"}}{Displays the silhouette plot (if available).}
\item{\code{"network"}}{Displays the network plot (if available).}
\item{\code{"cmds"}}{Displays the CMDs plot (if available).}
}
The default is \code{"all"}.}

\item{...}{arguments to be passed to methods}
}
\value{
This function displays the specified plot(s) in the current graphical device.
}
\description{
This function generates visualizations for an object of class \code{conceptMap}.
Depending on the method used to create the concept map, different types of plots
(e.g., heatmap, silhouette, network, or CMDS plots) are available.
}
\details{
The function behaves differently depending on the method used to create
the concept map. The \code{conceptMap} object must include attributes such as
\code{method} (e.g., "kmeans", "network", or "cmds") and the corresponding plot
objects (e.g., \code{heatmapPlot}, \code{silhouettePlot}, etc.).

The following methods are supported:
\itemize{
\item \code{"kmeans"}: Supports \code{"heatmap"} and \code{"silhouette"} plots.
\item \code{"network"}: Supports \code{"heatmap"} and \code{"network"} plots.
\item \code{"cmds"}: Supports \code{"heatmap"}, \code{"silhouette"}, and \code{"cmds"} plots.
}
}
\examples{
# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))

# Subject the data to sorter cluster analysis
myCMDataBySorters <- sorterMapping(myCMData)

# Subject sorter cluster 3 to concept mapping using default "network" method
myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])

# Visualise the concept map
plot(myCMAnalysis3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conceptMapping.R
\name{conceptMapping}
\alias{conceptMapping}
\title{Perform Concept Mapping Analysis}
\usage{
conceptMapping(
  CMData,
  method = "network",
  numberOfClusters = "auto",
  verbose = TRUE,
  rangeNumberOfClusters = 2:15,
  graph = FALSE,
  numberOfKmeansRestarts = 100,
  backgroundColor = "black",
  main = NULL,
  ...
)
}
\arguments{
\item{CMData}{A data frame containing concept mapping data.
It must include the columns:
\code{"sorterID"}, \code{"statement"}, and \code{"stackID"}.}

\item{method}{A string specifying the clustering method to use. Options are:
\code{"kmeans"}, \cr \code{"network"}, or \code{"cmds"}. Default is \code{"network"}.}

\item{numberOfClusters}{Either a character string (\code{"auto"}) to determine the optimal number of clusters
or an integer specifying the desired number of clusters. Default is \code{"auto"}.}

\item{verbose}{Logical, if \code{TRUE}, additional information about the processing steps is printed to the console.}

\item{rangeNumberOfClusters}{A vector of integers specifying the range of clusters to evaluate
when \cr \code{numberOfClusters = "auto"}. Default is \code{2:15}.}

\item{graph}{Logical. If \code{TRUE}, visualizes clustering results, including heatmaps and cluster-specific plots.
Default is \code{FALSE}.}

\item{numberOfKmeansRestarts}{An integer specifying the number of restarts for k-means clustering.
Only relevant if \code{method = "kmeans"}. Default is \code{100}.}

\item{backgroundColor}{A string specifying the background color of network plots. Default is \code{"black"}.}

\item{main}{A string specifying the title for plots. Default is \code{NULL}.}

\item{...}{Additional arguments, such as \code{resolution}, which may be passed to specific clustering methods.}
}
\value{
An object of class \code{conceptMap}, containing:
\describe{
\item{\code{allStatements}}{A data frame with statement numbers and text.}
\item{\code{CMData}}{The original concept mapping data.}
\item{\code{method}}{The clustering method used.}
\item{\code{numberOfClusters}}{The number of clusters identified.}
\item{\code{clusterResults}}{A vector indicating cluster assignments for each statement.}
\item{\code{heatmapPlot}}{A heatmap visualizing co-occurrence patterns.}
\item{\code{silhouettePlot}}{(If applicable) A silhouette plot for \code{"kmeans"} or \code{"cmds"}.}
\item{\code{networkPlot}}{(If applicable) A network plot for \code{"network"}.}
\item{\code{cmdsPlot}}{(If applicable) A CMDS plot for \code{"cmds"}.}
}
}
\description{
This function conducts a concept mapping analysis on sorter data, producing a concept map
based on one of three clustering methods: k-means, network analysis, or classical multidimensional scaling (CMDS).
}
\details{
The function supports three methods for clustering:
\itemize{
\item \code{"kmeans"}: Uses k-means clustering with an optional silhouette-based determination of cluster count.
\item \code{"network"}: Generates a network plot using modularity-based clustering.
\item \code{"cmds"}: Applies classical multidimensional scaling (CMDS) and clusters the results.
}

Heatmaps are created for all methods, while additional visualizations depend on the chosen method and \code{graph} parameter.
}
\examples{
# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))

# Subject the data to sorter cluster analysis
myCMDataBySorters <- sorterMapping(myCMData)

# Concept mapping on sorter cluster 3 using default "network" method
myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])

# Concept mapping using default network method using 3 clusters
myCMAnalysis3b <- conceptMapping(myCMDataBySorters[[3]], numberOfCluster = 3)

# Concept mapping using kmeans clustering and 3 clusters
myCMAnalysis3c <- conceptMapping(myCMDataBySorters[[3]], method = "kmeans",
  numberOfCluster = 3)

}
\references{
Kampen, J.K., Hageman, J.A., Breuer, M., & Tobi, H. (2025). The validity of concept mapping: let's call a spade a spade. Qual Quant. \doi{10.1007/s11135-025-02351-z}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/checkConceptMapData.R
\name{checkConceptMapData}
\alias{checkConceptMapData}
\title{Validate Concept Map Data}
\usage{
checkConceptMapData(CMData)
}
\arguments{
\item{CMData}{A data frame containing concept map data. This data frame must include
the required columns: "sorterID", "statement", and "stackID".}
}
\value{
Returns \code{TRUE} if all checks pass. If any check fails, an error is raised
with a descriptive message.
}
\description{
Checks whether the provided data frame meets the requirements for concept map data.
}
\details{
This function performs the following checks on the input data:
\itemize{
\item Verifies that \code{CMData} is a data frame.
\item Ensures the presence of required columns: \code{"sorterID"}, \code{"statement"}, and \code{"stackID"}.
\item Confirms that there are at least 2 unique values in the \code{"stackID"} column.
\item Confirms that there are at least 2 unique values in the \code{"sorterID"} column.
\item Confirms that there are at least 2 unique values in the \code{"statement"} column.
}
}
\examples{
# Example of valid data
validData <- data.frame(
  sorterID = c("resp1", "resp1", "resp1", "resp2",
   "resp2", "resp2", "resp3", "resp3", "resp3"),
  statement = c("London", "Frankfurt", "Berlin", "London",
  "Frankfurt", "Berlin", "London", "Frankfurt", "Berlin"),
  stackID = c("capital city", "city", "capital city", 1, 2, 2, "A", "B", "A")
)
checkConceptMapData(validData) # Should return TRUE

# Example of invalid data (missing columns)
invalidData <- data.frame(
  sorterID = c(1, 2),
  stackID = c(1, 2)
)
# checkConceptMapData(invalidData) # Will return False as an error

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.conceptMap.R
\name{print.conceptMap}
\alias{print.conceptMap}
\title{Print a Concept Map Object}
\usage{
\method{print}{conceptMap}(x, whichCluster = NULL, ...)
}
\arguments{
\item{x}{An object of class \code{"conceptMap"} containing the results of concept mapping analysis
creating using the function \code{\link[cmAnalysis]{conceptMapping}}.}

\item{whichCluster}{A vector of cluster numbers to display. If \code{NULL} (default), all clusters are displayed.}

\item{...}{arguments to be passed to methods}
}
\value{
This function does not return a value; it prints the details of the concept map to the console.
}
\description{
This function prints a detailed summary of a concept map object, including information about
statements, sorters, clusters, and the specific statements within selected clusters.
}
\details{
The function first checks if the input object is of class \code{"conceptMap"} and validates the
requested clusters (if specified). It provides an overview of the number of statements, sorters,
and clusters. For each requested cluster, the function lists the statements included in that cluster.
}
\examples{
# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))

# Subject the data to sorter cluster analysis
myCMDataBySorters <- sorterMapping(myCMData)

# Subject sorter cluster 3 to concept mapping using default "network" method
myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])

# Print content of concept map of sorter cluster 3
print(myCMAnalysis3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/crossClusterMap.R
\name{crossClusterMap}
\alias{crossClusterMap}
\title{Cross-Cluster Mapping Between Concept Maps}
\usage{
crossClusterMap(conceptMap1, conceptMap2)
}
\arguments{
\item{conceptMap1}{An object of class \code{"conceptMap"} representing the first concept map.}

\item{conceptMap2}{An object of class \code{"conceptMap"} representing the second concept map.}
}
\value{
The function does not return a value but generates a \code{ggplot2} visualization.
}
\description{
This function compares two concept maps by aligning their clustering results
and visualizing the correspondence between clusters. It identifies matches
between clusters from the two maps and highlights differences visually.
}
\details{
The function aligns clusters between two concept maps using an optimal matching
algorithm. It first creates a matching matrix based on the overlap between clusters
in the two maps. Then, it uses the Hungarian algorithm (via the \code{solve_LSAP} function
from the \code{clue} package) to find an optimal alignment of clusters.

The output is a plot that shows the alignment of clusters from the two concept maps,
with connecting lines colored to indicate matches or mismatches. Statements not
clustered in both maps are highlighted in grey.
}
\examples{
# Simulate data with custom parameters:
set.seed(1)
myCMData <- simulateCardData(nSorters=40, pCorrect=.90, attributeWeights=c(1,1,1,1))

# Subject the data to sorter cluster analysis
myCMDataBySorters <- sorterMapping(myCMData)

# Subject sorter cluster 1 to concept mapping using default "network" method
myCMAnalysis1 <- conceptMapping(myCMDataBySorters[[1]])

# Subject sorter cluster 3 to concept mapping using default "network" method
myCMAnalysis3 <- conceptMapping(myCMDataBySorters[[3]])

# Visualise comparison of results of two sorter clusters
crossClusterMap(myCMAnalysis1, myCMAnalysis3)

}
\seealso{
\code{\link[clue]{solve_LSAP}}, \code{\link[ggplot2]{ggplot}}
}
